"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useField = void 0;

var _react = _interopRequireWildcard(require("react"));

var _get = _interopRequireDefault(require("lodash/get"));

var _set = _interopRequireDefault(require("lodash/set"));

var _validate = _interopRequireDefault(require("./validate"));

var _form = require("./form");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const useField = ({
  name,
  rules: fieldRules
}) => {
  const {
    errors,
    values,
    setErrors,
    hasBlurred,
    updateValue,
    valuesBlurred,
    rules
  } = _react.default.useContext(_form.FormContext); //set the rules when they change, clear when unmounted


  (0, _react.useEffect)(() => {
    //This check lets us pass in a single rule without making an array, or an array of rules
    if (Array.isArray(fieldRules)) {
      rules.current[name] = fieldRules;
    } else {
      rules.current[name] = fieldRules ? [fieldRules] : [];
    }

    return () => {
      rules.current[name] = [];
    };
  }, [fieldRules]);
  let value = (0, _get.default)(values, name); // Pulling out just this field's errors

  let fieldErrors = errors.filter(error => error.name === name).map(error => error.message); //Args needed for validation

  let validationProps = {
    values,
    rules,
    valuesBlurred,
    setErrors,
    errors
  };

  const handleChange = (value, field) => {
    /*
      If this field 
      - has errors
      - blurred before
      - has dependant rules
      check validation on change
    */
    if (fieldErrors.length || valuesBlurred[field || name]) {
      let newValues = { ...values
      };
      (0, _set.default)(newValues, field || name, value);
      (0, _validate.default)({ ...validationProps,
        values: newValues,
        valuesBlurred: { ...valuesBlurred,
          [field || name]: true
        }
      });
    }

    updateValue(field || name, value);
  };
  /*
    When blurring for the first time, 
    lets send hasBlurred, and validate the field
  */


  const handleBlur = () => {
    hasBlurred(name);
    (0, _validate.default)({ ...validationProps,
      valuesBlurred: { ...valuesBlurred,
        [name]: true
      }
    });
  };

  return {
    handleBlur,
    handleChange,
    value,
    values,
    errors: fieldErrors.length ? fieldErrors : null
  };
};

exports.useField = useField;